@@ -6,8 +6,8 @@ from pkg_resources import resource_filename
 from logging import info, ERROR, WARNING, INFO, DEBUG, CRITICAL, basicConfig, error
 from uno import createUnoStruct
 
-__version__ = '0.2.9999'
-__versiondatetime__=datetime(2021, 9, 29, 12, 44)
+__version__ = '0.3.0'
+__versiondatetime__=datetime(2021, 10, 1, 21, 44)
 __versiondate__=__versiondatetime__.date()
 
 try:
@@ -399,3 +399,10 @@ def date2localc1989(o):
 def time2localc1989(o):
     seconds=o.hour*3600+o.minute*60+o.second
     return float(seconds) / 86400
+    
+## Used to change port when there are multiple sockets of libreoffice accepting
+def next_port(last,  first_port,  instances):
+    if last==first_port+instances -1:
+        return first_port
+    else:
+        return last+1
